/* iplog.h of iplog v1.3 -- eric@ojnk.nu */
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <pwd.h>
#include <grp.h>
#include <fcntl.h>
#include <errno.h>
extern int errno;

#include "config.h"

#ifndef DROP_USER
#define DROP_USER "nobody"
#endif

#ifndef DROP_GROUP
#define DROP_GROUP "nogroup"
#endif

#ifndef LOGGING_OPTIONS
#define LOGGING_OPTIONS	LOG_PID
#endif

#ifndef LOGGING_FACILITY
#define LOGGING_FACILITY LOG_DAEMON
#endif

#ifndef LOGGING_PRIORITY
#define LOGGING_PRIORITY	LOG_NOTICE
#endif

#define LEVEL	LOGGING_PRIORITY

void fatal(const char *fmt,...)
{
	char *p = malloc(512);
	va_list ap;

	if (p == NULL)
		exit(-1);

	va_start(ap, fmt);
	(void)vsnprintf(p, 512, fmt, ap);
	va_end(p);

	syslog(LOG_WARNING, p);
	free(p);

	exit(-1);
}


void fork_to_background(void)
{
	switch (fork()) {
		case -1:
			_exit(0);
		case 0:
			break;
		default:
			_exit(0);
	}

	if (setsid() == -1)
		fatal("setsid: %m");

	chdir("/");
	umask(022);

	(void)close(0);				/* stdin */
	(void)close(1);				/* stdout */
	(void)close(2);				/* stderr */

	return;
}

void assert_id(uid_t uid, gid_t gid)
{
	struct passwd *pw;
	struct group *gr;
	uid_t my_uid;
	gid_t my_gid;

	if ((pw = getpwnam(DROP_USER)) == NULL)
		my_uid = 65534;
	else
		my_uid = pw->pw_uid;

	if ((gr = getgrnam(DROP_GROUP)) == NULL)
		my_uid = 65534;
	else if (pw != NULL)
		my_uid = pw->pw_gid;
	else
		my_uid = gr->gr_gid;

	uid = my_uid;
	gid = my_gid;

	return;
}

void drop_privileges(void)
{
	gid_t gid = -2;
	uid_t uid = -2;

	assert_id(uid, gid);

	if (setregid(gid, gid) < 0)
		fatal("setregid: %m");

	if (setgroups(1, &gid) < 0)
		fatal("setgroups: %m");

	if (setreuid(uid, uid) < 0)
		fatal("setreuid: %m");

	return;
}
